package it.unipr.ce.dsg.deus.example.basic;

import java.util.Properties;

import it.unipr.ce.dsg.deus.core.Event;
import it.unipr.ce.dsg.deus.core.InvalidParamsException;
import it.unipr.ce.dsg.deus.core.Node;
import it.unipr.ce.dsg.deus.core.Process;
import it.unipr.ce.dsg.deus.core.RunException;
import it.unipr.ce.dsg.deus.p2p.node.Peer;
//import it.unipr.ce.dsg.deus.util.Distributions;

public class MinAgeConnectionEvent extends Event {

	private static final String IS_BIDIRECTIONAL = "isBidirectional";
	private static final String MAX_NUM_INITIAL_CONNECTIONS = "maxNumInitialConnections";
	
	private boolean isBidirectional = false;
	private int maxNumInitialConnections = 0;
	
	public MinAgeConnectionEvent(String id, Properties params,
			Process parentProcess) throws InvalidParamsException {
		super(id, params, parentProcess);
		initialize();
	}
	
	public void initialize() throws InvalidParamsException {
		if (params.containsKey(IS_BIDIRECTIONAL))
			isBidirectional = Boolean.parseBoolean(params.getProperty(IS_BIDIRECTIONAL)); 
		if (params.containsKey(MAX_NUM_INITIAL_CONNECTIONS))
			maxNumInitialConnections = Integer.parseInt(params.getProperty(MAX_NUM_INITIAL_CONNECTIONS));
	}
	
	public Object clone() {
		MinAgeConnectionEvent clone = (MinAgeConnectionEvent) super.clone();
		return clone;
	}

	@Override
	public void run() throws RunException {
		
		if (getParentProcess() == null)
			throw new RunException(
					"A parent process must be set in order to run "
							+ getClass().getCanonicalName());
		// create a node (the type is randomly chosen among those which are
		// associated to the process)
		Node node = (Node) getParentProcess().getReferencedNodes().get(
				this.engine.getSimulationRandom().nextInt(
						getParentProcess().getReferencedNodes().size()))
				.createInstance(this.engine);
		this.engine.addNode(node);
		associatedNode = node;
				
		if (!(associatedNode instanceof Peer))
			throw new RunException("The associated node is not a Peer!");
		
		Peer peer = (Peer) associatedNode;
		
		int n = this.engine.getNodes().size();
		
		attachToYoungestSupernodes(n, maxNumInitialConnections, peer);
		System.out.println("CONN " + peer + " has k = " + peer.getNeighbors().size());
	}
	
	private void attachToYoungestSupernodes(int n, int m, Peer peer) {
		//System.out.println("> peer node degree before = " + peer.getNeighbors().size());
		Peer target = null;
		//int k = 0;
		//System.out.println("peer = " + peer.getKey());
		//System.out.println("n = " + n);
		if (n > m) {
		for (int i = 1; i <= m; i++) {
			target = (Peer) this.engine.getNodes().get(n-1-i);
			//System.out.println("target = " + target.getKey());
			/*
			if (target.getNeighbors().size() < m)
				System.out.println("target has k = " + target.getNeighbors().size());
			*/
			//System.out.println("target node degree before connection = " + target.getNeighbors().size());
			if (!target.getNeighbors().contains(peer)) {
				peer.addNeighbor(target);
				if (isBidirectional) {
					target.addNeighbor(peer);
					System.out.println(target + " has k = " + target.getNeighbors().size());
				}
				//k++;
			}
			//System.out.println("target node degree after connection = " + target.getNeighbors().size());
			//System.out.println("k = " + k);
			//if (k == m)
				//break;
		}
		}
		//System.out.println("peer node degree after = " + peer.getNeighbors().size());
	}

	public int getNumInitialConnections() {
		return maxNumInitialConnections;
	}

	public void setNumInitialConnections(int numInitialConnections) {
		this.maxNumInitialConnections = numInitialConnections;
	}
}
