package it.unipr.ce.dsg.deus.impl.event;

import it.unipr.ce.dsg.deus.core.InvalidParamsException;
import it.unipr.ce.dsg.deus.core.Node;
import it.unipr.ce.dsg.deus.core.Event;
import it.unipr.ce.dsg.deus.core.Process;
import it.unipr.ce.dsg.deus.core.RunException;

import java.util.Properties;

/**
 * This event represents the birth of a simulation node. During its execution an
 * instance of the node associated to the event will be created.
 * 
 * @author Matteo Agosti (agosti@ce.unipr.it)
 * @author Michele Amoretti (michele.amoretti@unipr.it)
 * 
 */
public class BirthEvent extends Event {

	public BirthEvent(String id, Properties params, Process parentProcess)
			throws InvalidParamsException {
		super(id, params, parentProcess);
		initialize();
	}

	public void initialize() throws InvalidParamsException {
		// TODO implement params to establish the policy of node type initialization
		// (now choose one type only - randomly - from the parent process)
	}

	public void run() throws RunException {

		if (getParentProcess() == null)
			throw new RunException(
					"A parent process must be set in order to run "
							+ getClass().getCanonicalName());
		// create a node (the type is randomly chosen among those which are
		// associated to the process)
		Node n = (Node) getParentProcess().getReferencedNodes().get(
				this.engine.getSimulationRandom().nextInt(
						getParentProcess().getReferencedNodes().size()))
				.createInstance(this.engine);
		this.engine.addNode(n);
		associatedNode = n;
	}

}
