package it.unipr.ce.dsg.deus.util;

/*
 * Copyright (C) 2004 John D Lamb (J.D.Lamb@btinternet.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* The only real purpose of this class is to give t-values for right tail
   areas of the t distribution. Rather than write an algorithm for incomplete
   Beta functions, we just autogenerate this list from a python script */
public class T {
    
    private static boolean check3( double x, double y ){
	return java.lang.Math.round( (x - y) * 1000 ) == 0;
    }

    public static double tVal( double tail, int degreesOfFreedom ){
	if( degreesOfFreedom < 1 )
	    return 0;
	if( degreesOfFreedom > 500 )
	    degreesOfFreedom = 500;
	int col;
	if( check3( tail, 0.25 ) )
	    col = 0;
	else if( check3( tail, 0.1 ) )
	    col = 1;
	else if( check3( tail, 0.05 ) )
	    col = 2;
	else if( check3( tail, 0.025 ) )
	    col = 3;
	else if( check3( tail, 0.01 ) )
	    col = 4;
	else if( check3( tail, 0.005 ) )
	    col = 5;
	else
	    return 0;
	return t[degreesOfFreedom][col];
    }

    public static double tGood( double tail, int degreesOfFreedom ){
	if( degreesOfFreedom < 1 )
	    return 0;
	if( degreesOfFreedom > 120 )
	    return zGood( tail );
	if( degreesOfFreedom >= 30 && degreesOfFreedom < 35 )
	    degreesOfFreedom = 30;
	if( degreesOfFreedom >= 35 && degreesOfFreedom < 50 )
	    degreesOfFreedom = 40;
	if( degreesOfFreedom >= 50 && degreesOfFreedom < 120 )
	    degreesOfFreedom = 60;
	int col;
	if( check3( tail, 0.25 ) )
	    col = 0;
	else if( check3( tail, 0.1 ) )
	    col = 1;
	else if( check3( tail, 0.05 ) )
	    col = 2;
	else if( check3( tail, 0.025 ) )
	    col = 3;
	else if( check3( tail, 0.01 ) )
	    col = 4;
	else if( check3( tail, 0.005 ) )
	    col = 5;
	else
	    return 0;
	double d = t[degreesOfFreedom][col] * 100;
	long l = java.lang.Math.round( d );
	return (double)l / 100;
    }

    public static double zVal( double tail ){
	int col;
	if( check3( tail, 0.25 ) )
	    col = 0;
	else if( check3( tail, 0.1 ) )
	    col = 1;
	else if( check3( tail, 0.05 ) )
	    col = 2;
	else if( check3( tail, 0.025 ) )
	    col = 3;
	else if( check3( tail, 0.01 ) )
	    col = 4;
	else if( check3( tail, 0.005 ) )
	    col = 5;
	else
	    return 0;
	return z[col];
    }

    public static double zGood( double tail ){
	double d = zVal( tail ) * 100;
	long l = java.lang.Math.round( d );
	return (double)l / 100;
    }


    public static void main( java.lang.String[] args ){
	System.out.println( tVal( 0.01, 20 ) );
	System.out.println( tVal( 0.01, 500 ) );
    }

    private static double[] z
	= { 0.674189140, 1.281728757, 1.645211440, 1.960394917, 2.326785333,
	    1.645211440 };

    private static double t[][] = {
	{ 0, 0, 0, 0, 0, 0 },
	{ 0.999999998754596, 3.07768353681211, 6.31375150233019,
	  12.7062047395164, 31.8205159624801, 63.6567408355085 },
	{ 0.816496580927727, 1.88561808316415, 2.91998558035379,
	  4.30265272975056, 6.96455673429069, 9.92484320094496 },
	{ 0.764892328326885, 1.6377443524111, 2.35336343469542,
	  3.18244630507878, 4.54070285690172, 5.84090930974892 },
	{ 0.740697084053873, 1.53320627271965, 2.13184678203492,
	  2.77644509837475, 3.74694738792736, 4.60409486738959 },
	{ 0.726686844478852, 1.47588404791287, 2.01504837220353,
	  2.57058183481906, 3.36492999748551, 4.03214298350002 },
	{ 0.717558196431747, 1.43975574202069, 1.94318027439818,
	  2.44691184654141, 3.14266839575865, 3.7074280205117 },
	{ 0.71114177722066, 1.41492392534731, 1.89457860375594,
	  2.36462425104947, 2.99795156646347, 3.49948329403363 },
	{ 0.706386611136738, 1.3968153026426, 1.8595480333977,
	  2.30600413339359, 2.89645944630992, 3.3553873312366 },
	{ 0.702722144749712, 1.38302873559789, 1.83311293175028,
	  2.26215715826384, 2.82143792150159, 3.24983554122325 },
	{ 0.699812058936665, 1.37218363467518, 1.81246112065477,
	  2.22813885139066, 2.76376945787456, 3.16927267170031 },
	{ 0.697445324897947, 1.36343031553732, 1.79588481432048,
	  2.20098515880618, 2.71807918326028, 3.10580651367074 },
	{ 0.695482862628029, 1.35621732919697, 1.7822875546441,
	  2.17881282724733, 2.68099799204012, 3.05453958602996 },
	{ 0.693829301177257, 1.35017128690485, 1.7709333941454,
	  2.16036865232915, 2.65030883603077, 3.01227583853162 },
	{ 0.692417066372763, 1.34503037118015, 1.76131013268074,
	  2.14478668728136, 2.62449406457174, 2.97684273403002 },
	{ 0.691196945648942, 1.34060560251923, 1.75305035082804,
	  2.1314495445131, 2.60248029046493, 2.94671288291026 },
	{ 0.69013225040924, 1.33675715912097, 1.74588367506518,
	  2.1199052976103, 2.58348718482025, 2.92078162155692 },
	{ 0.689195071676805, 1.33337938649824, 1.73960672424216,
	  2.10981557547777, 2.56693398303576, 2.89823051841579 },
	{ 0.688363802925825, 1.33039093883312, 1.73406360396673,
	  2.10092203693637, 2.55237962919317, 2.87844047098343 },
	{ 0.687621456610355, 1.32772820232136, 1.72913280782827,
	  2.09302404992931, 2.53948318926138, 2.86093460411132 },
	{ 0.686954492810035, 1.32534070427198, 1.72471823793465,
	  2.08596344643447, 2.5279770009238, 2.84533970671426 },
	{ 0.686351987049586, 1.3231878701324, 1.72074289625815,
	  2.07961384360464, 2.51764801368694, 2.83135955412215 },
	{ 0.68580502801129, 1.32123673661431, 1.71714437256162,
	  2.07387306642907, 2.50832454991182, 2.81875606023846 },
	{ 0.685306274321937, 1.31946023327584, 1.71387152538037,
	  2.06865760852751, 2.49986673578571, 2.80733568329929 },
	{ 0.684849623472569, 1.31783592528962, 1.71088207689396,
	  2.06389855925126, 2.49215946863293, 2.7969395041767 },
	{ 0.684429961117933, 1.31634506914602, 1.70814075748008,
	  2.05953854981945, 2.48510717478402, 2.78743581293473 },
	{ 0.684042968877252, 1.31497185980801, 1.70561791511742,
	  2.05552943507755, 2.47862982281675, 2.77871453242255 },
	{ 0.683684975308364, 1.31370290727451, 1.70328844009148,
	  2.05183051220731, 2.4726599110152, 2.77068295603446 },
	{ 0.683352839150721, 1.31252677477683, 1.70113092752341,
	  2.04840713673686, 2.46714009684074, 2.76326245417343 },
	{ 0.683043856970346, 1.31143363904553, 1.69912702446581,
	  2.04522964100022, 2.46202135881766, 2.75638590216217 },
	{ 0.682755689457999, 1.31041502177866, 1.69726088412051,
	  2.04227245495389, 2.45726154084258, 2.74999565182003 },
	{ 0.682486302128482, 1.30946354514743, 1.69551877961967,
	  2.0395134448116, 2.45282419159937, 2.7440419172893 },
	{ 0.682233917242462, 1.3085727879542, 1.69388874495635,
	  2.03693334161428, 2.44867763160373, 2.73848147973117 },
	{ 0.681996974548701, 1.3077371183494, 1.69236030505228,
	  2.03451529531913, 2.444794197455, 2.73327663977567 },
	{ 0.681774099015074, 1.30695157999669, 1.69092425060765,
	  2.03224450687939, 2.4411496252495, 2.72839436418063 },
	{ 0.68156407413889, 1.30621179375239, 1.68957245254876,
	  2.03010792548014, 2.43772254416329, 2.72380558598936 },
	{ 0.681365819742621, 1.30551387602993, 1.68829770818152,
	  2.02809399785473, 2.43449405790904, 2.71948462688612 },
	{ 0.681178373399644, 1.30485437714639, 1.68709361290545,
	  2.02619245952426, 2.43144739678331, 2.7154087176252 },
	{ 0.681000874816056, 1.30423019889685, 1.68595445791098,
	  2.02439416000467, 2.42856762680062, 2.71155759761195 },
	{ 0.680832552633966, 1.30363858292763, 1.68487511916327,
	  2.02269091570404, 2.42584140528332, 2.70791318290853 },
	{ 0.680672713229227, 1.30307704615457, 1.68385101047441,
	  2.02107538552542, 2.4232567744721, 2.70445926675787 },
	{ 0.680520731160781, 1.30254335168156, 1.68287799893694,
	  2.01954096519021, 2.42080299082261, 2.70118130283359 },
	{ 0.680376040994094, 1.30203547903027, 1.68195235391598,
	  2.01808169707526, 2.41847035863643, 2.69806618540143 },
	{ 0.680238130273811, 1.30155159858738, 1.68107069927391,
	  2.01669219769336, 2.4162501276681, 2.69510207826273 },
	{ 0.680106533461103, 1.30109004958867, 1.68022997224524,
	  2.01536757275701, 2.41413436697361, 2.69227826472023 },
	{ 0.679980826684992, 1.30064932742408, 1.67942738790614,
	  2.01410338703434, 2.41211587440317, 2.68958501831842 },
	{ 0.679860623182903, 1.30022804234944, 1.6786604083704,
	  2.01289559690574, 2.41018809479285, 2.68701349110232 },
	{ 0.679745569326822, 1.29982494138877, 1.67792671599445,
	  2.01174051154171, 2.40834504892237, 2.68455561664029 },
	{ 0.67963534114965, 1.29943887214869, 1.67722418999554,
	  2.01063475525477, 2.40658127163804, 2.68220402563352 },
	{ 0.679529641299333, 1.29906877759059, 1.67655088598651,
	  2.0095752345714, 2.40489175778001, 2.67995197222301 },
	{ 0.679428196360923, 1.29871368636833, 1.67590501801141,
	  2.00855910934773, 2.40327191479321, 2.67779326943721 },
	{ 0.679330754495465, 1.29837270431762, 1.67528494273238,
	  2.00758376736093, 2.4017175210764, 2.67572223250769 },
	{ 0.679237083352999, 1.29804500693971, 1.67468915085853,
	  2.0066468018984, 2.4002246892798, 2.67373362894598 },
	{ 0.679146968223149, 1.29772983274676, 1.67411623361092,
	  2.00574599193998, 2.3987898338697, 2.67182263443908 },
	{ 0.679060210392465, 1.29742648321524, 1.67356490302656,
	  2.00487928458937, 2.39740964240063, 2.66998479383049 },
	{ 0.678976625681941, 1.29713429451733, 1.67303396172227,
	  2.00404477946376, 2.39608105000623, 2.66821598647652 },
	{ 0.678896043141931, 1.29685266673684, 1.67252229925745,
	  2.00324071478999, 2.39480121669683, 2.66651239543815 },
	{ 0.678818303885489, 1.29658103748711, 1.67202888438398,
	  2.00246545499509, 2.3935675071118, 2.66487048001635 },
	{ 0.678743260042821, 1.29631888361401, 1.67155275811082,
	  2.00171747960576, 2.39237747241275, 2.66328695120205 },
	{ 0.678670773822648, 1.29606571783362, 1.67109302748477,
	  2.00099537330018, 2.39122883407672, 2.66175874971557 },
	{ 0.67860071666797, 1.29582108570986, 1.67064886000253,
	  2.00029781697263, 2.39011946934249, 2.66028302629161 },
	{ 0.67853296849535, 1.29558456293256, 1.67021947858092,
	  1.99962357969472, 2.38904739812574, 2.65885712397852 },
	{ 0.678467417008008, 1.29535575286194, 1.66980415702095,
	  1.99897151147011, 2.38801077123216, 2.65747856216046 },
	{ 0.678403957074802, 1.29513428430922, 1.66940221591012,
	  1.99834053668993, 2.38700785971945, 2.65614502219152 },
	{ 0.67834249016774, 1.29491980952757, 1.6690130189145,
	  1.99772964821508, 2.38603704527991, 2.65485433438224 },
	{ 0.678282923851415, 1.29471200239042, 1.66863596941815,
	  1.99713790201353, 2.38509681153134, 2.65360446623449 },
	{ 0.678225171319464, 1.29451055673695, 1.66827050747152,
	  1.99656441695156, 2.38418573611785, 2.65239351176011 },
	{ 0.678169150972381, 1.29431519095244, 1.66791610701799,
	  1.99600835192708, 2.38330248353606, 2.65121968179493 },
	{ 0.678114786032995, 1.2941256225715, 1.6675722733674,
	  1.99546892923243, 2.38244579860103, 2.65008129518372 },
	{ 0.678062004195224, 1.29394160260192, 1.66723854089315,
	  1.99494541280898, 2.38161450050689, 2.64897677075548 },
	{ 0.678010737303634, 1.29376289097122, 1.66691447092836,
	  1.99443710937038, 2.38080747738954, 2.64790461999578 },
	{ 0.677960921059464, 1.29358926125988, 1.66659965501693,
	  1.9939433653408, 2.38002368137311, 2.6468634403594 },
	{ 0.677912494751937, 1.2934204997442, 1.66629369266255,
	  1.99346356405145, 2.37926212805417, 2.64585190915618 },
	{ 0.677865401011221, 1.29325640451895, 1.66599622014202,
	  1.99299712317239, 2.37852187637956, 2.64486877794549 },
	{ 0.677819585581885, 1.29309678469148, 1.66570688894121,
	  1.99254349235495, 2.37780204859504, 2.64391286740019 },
	{ 0.677774997114319, 1.29294145964106, 1.66542536936345,
	  1.99210215106647, 2.3771018110267, 2.64298306258935 },
	{ 0.677731586972797, 1.29279025833717, 1.66515134927644,
	  1.99167260659773, 2.37642037475964, 2.64207830864229 },
	{ 0.677689309058484, 1.29264301871141, 1.66488453295807,
	  1.99125439222914, 2.37575699267477, 2.64119761048797 },
	{ 0.6776481196463, 1.29249958707784, 1.66462464003189,
	  1.99084706553887, 2.37511095671759, 2.64034001436062 },
	{ 0.677607977233906, 1.29235981759829, 1.66437140448461,
	  1.99045020684259, 2.37448159537586, 2.63950462649166 },
	{ 0.67756884240227, 1.29222357178757, 1.66412457375801,
	  1.99006341775109, 2.37386827135594, 2.63869059535142 },
	{ 0.677530677686664, 1.29209071805588, 1.66388390790875,
	  1.98968631983664, 2.37327037942819, 2.63789711239339 },
	{ 0.677493447457014, 1.29196113718367, 1.66364917883064,
	  1.98931855339848, 2.37268734443603, 2.6371234093668 },
	{ 0.67745711780686, 1.29183469855637, 1.66342016953356,
	  1.98895977631822, 2.37211861945185, 2.63636875584924 },
	{ 0.677421656450403, 1.29171129452721, 1.66319667347446,
	  1.98860966299909, 2.37156368406639, 2.63563245693369 },
	{ 0.677387032626999, 1.29159081714518, 1.66297849393614,
	  1.98826790337997, 2.37102204279962, 2.63491385110966 },
	{ 0.677353217011848, 1.29147316351952, 1.66276544344992,
	  1.98793420202039, 2.37049322362749, 2.63421230826907 },
	{ 0.677320181633528, 1.29135823552642, 1.66255734325827,
	  1.98760827724819, 2.36997677660792, 2.63352722787391 },
	{ 0.677287899796437, 1.2912459395364, 1.66235402281489,
	  1.98728986036742, 2.36947227260786, 2.63285803723939 },
	{ 0.677256346009147, 1.29113618615968, 1.66215531931842,
	  1.98697869491891, 2.36897930211077, 2.6322041899307 },
	{ 0.677225495917291, 1.29102889000896, 1.66196107727772,
	  1.98667453599209, 2.36849747411042, 2.63156516428666 },
	{ 0.67719532624053, 1.29092396947802, 1.66177114810643,
	  1.98637714958211, 2.36802641507647, 2.6309404620255 },
	{ 0.67716581471452, 1.29082134653442, 1.66158538974335,
	  1.98608631198908, 2.36756576798883, 2.63032960695271 },
	{ 0.67713694003548, 1.29072094652641, 1.66140366629883,
	  1.98580180925774, 2.36711519143559, 2.62973214374761 },
	{ 0.677108681809451, 1.29062269800141, 1.6612258477227,
	  1.98552343665196, 2.36667435876918, 2.62914763683387 },
	{ 0.677081020504041, 1.29052653253668, 1.66105180949413,
	  1.98525099816393, 2.36624295731666, 2.62857566932355 },
	{ 0.677053937403257, 1.29043238458041, 1.66088143233101,
	  1.98498430605344, 2.36582068764057, 2.62801584201924 },
	{ 0.677027414565578, 1.29034019130261, 1.66071460191711,
	  1.98472318041717, 2.36540726284628, 2.62746777249107 },
	{ 0.677001434784133, 1.29024989245534, 1.66055120864637,
	  1.98446744878343, 2.36500240793211, 2.62693109420373 },
	{ 0.67697598154945, 1.2901614302415, 1.66039114738294,
	  1.98421694573281, 2.36460585917796, 2.62640545569582 },
	{ 0.676951039014624, 1.29007474919131, 1.66023432226084,
	  1.98397151254108, 2.3642173635764, 2.62589051982213 },
	{ 0.676926591962383, 1.28998979604641, 1.66008062648398,
	  1.98373099684385, 2.36383667828842, 2.62538596302041 },
	{ 0.67690262577377, 1.28990652551339, 1.65992997193855,
	  1.98349525232178, 2.36346357014306, 2.62489147464514 },
	{ 0.67687912639981, 1.28982487686682, 1.65978226924365,
	  1.98326413840266, 2.36309781515848, 2.62440675631773 },
	{ 0.676856080333, 1.28974480856085, 1.65963743248719,
	  1.98303751998306, 2.36273919809421, 2.62393152134123 },
	{ 0.676833474582414, 1.28966627515338, 1.6594953790586,
	  1.9828152671644, 2.36238751202859, 2.62346549412471 },
	{ 0.676811296648484, 1.28958923292821, 1.6593560294922,
	  1.98259725500468, 2.36204255796068, 2.62300840964522 },
	{ 0.676789534500605, 1.28951363981344, 1.65921930731863,
	  1.98238336773351, 2.36170414443687, 2.62256001295778 },
	{ 0.676768176554767, 1.28943945530528, 1.65908513892534,
	  1.98217348081336, 2.36137208719348, 2.62212005872591 },
	{ 0.676747211653613, 1.28936664039525, 1.65895345342392,
	  1.98196748718962, 2.3610462088227, 2.62168831074716 },
	{ 0.676726629046404, 1.28929515750227, 1.65882418252574,
	  1.98176527953289, 2.3607263384593, 2.62126454155802 },
	{ 0.676706418371094, 1.28922497040755, 1.65869726042351,
	  1.9815667544229, 2.36041231147981, 2.62084853202105 },
	{ 0.676686569636747, 1.28915604419365, 1.65857262367968,
	  1.98137181217149, 2.36010396921953, 2.62044007095754 },
	{ 0.676667073207056, 1.28908834518647, 1.65845021112087,
	  1.98118035665694, 2.35980115870866, 2.62003895477313 },
	{ 0.676647919784804, 1.28902184090044, 1.65832996373748,
	  1.98099229516509, 2.35950373241292, 2.61964498713122 },
	{ 0.676629100396991, 1.28895649998654, 1.65821182458917,
	  1.98080753824007, 2.35921154800237, 2.61925797863196 },
	{ 0.676610606380992, 1.28889229218295, 1.65809573871466,
	  1.98062599954193, 2.35892446811502, 2.61887774651509 },
	{ 0.676592429371002, 1.28882918826848, 1.65798165304675,
	  1.9804475957131, 2.35864236015118, 2.61850411434812 },
	{ 0.6765745612857, 1.28876716001804, 1.65786951633118,
	  1.9802722462492, 2.35836509606106, 2.61813691178508 },
	{ 0.676556994316106, 1.28870618016065, 1.65775927905005,
	  1.98009987337977, 2.35809255215461, 2.61777597428221 },
	{ 0.676539720914173, 1.28864622233953, 1.65765089334884,
	  1.97993040195184, 2.35782460891877, 2.61742114286418 },
	{ 0.676522733782095, 1.28858726107394, 1.65754431296714,
	  1.97976375932128, 2.35756115084151, 2.61707226390147 },
	{ 0.676506025862055, 1.28852927172316, 1.65743949317277,
	  1.97959987524895, 2.3573020662431, 2.61672918885094 },
	{ 0.676489590325998, 1.2884722304524, 1.65733639069946,
	  1.9794386818021, 2.35704724712288, 2.61639177409489 },
	{ 0.676473420567171, 1.28841611419957, 1.65723496368683,
	  1.97928011326009, 2.35679658900739, 2.61605988070062 },
	{ 0.676457510190432, 1.28836090064478, 1.65713517162418,
	  1.97912410602545, 2.35654999080509, 2.61573337425146 },
	{ 0.676441853004424, 1.28830656818031, 1.65703697529619,
	  1.97897059853865, 2.35630735467501, 2.61541212465532 },
	{ 0.676426443012979, 1.28825309588283, 1.656940336732,
	  1.97881953119728, 2.35606858589406, 2.61509600597642 },
	{ 0.676411274408035, 1.28820046925201, 1.65684521915567,
	  1.97867084627868, 2.35583359273477, 2.61478489627029 },
	{ 0.676396341562102, 1.28814865722741, 1.65675158693985,
	  1.97852448786646, 2.35560228634623, 2.6144786774244 },
	{ 0.676381639020941, 1.28809764644224, 1.65665940556151,
	  1.97838040178067, 2.35537458064549, 2.61417723501345 },
	{ 0.676367161497713, 1.28804741845304, 1.65656864155887,
	  1.97823853551044, 2.3551503922074, 2.6138804581518 },
	{ 0.676352903865708, 1.28799795537845, 1.65647926249166,
	  1.97809883815069, 2.35492964016593, 2.61358823935912 },
	{ 0.676338861153158, 1.28794923987728, 1.65639123690175,
	  1.97796126034078, 2.35471224611357, 2.61330047443196 },
	{ 0.67632502853691, 1.28790125512876, 1.65630453427679,
	  1.97782575420668, 2.35449813401023, 2.61301706231982 },
	{ 0.676311401336922, 1.28785398481305, 1.65621912501474,
	  1.97769227330543, 2.35428723009483, 2.61273790500372 },
	{ 0.676297975011429, 1.28780741309246, 1.65613498038997,
	  1.97756077257168, 2.35407946280025, 2.61246290738993 },
	{ 0.676284745151338, 1.2877615245942, 1.65605207252141,
	  1.97743120826759, 2.35387476267125, 2.61219197719113 },
	{ 0.676271707475583, 1.28771630439323, 1.65597037434162,
	  1.97730353793385, 2.35367306228969, 2.61192502483564 },
	{ 0.676258857827026, 1.28767173799579, 1.65588985956701,
	  1.97717772034332, 2.35347429619761, 2.61166196336022 },
	{ 0.676246192167023, 1.28762781132461, 1.65581050267034,
	  1.97705371545691, 2.35327840082809, 2.61140270831804 },
	{ 0.676233706572119, 1.28758451070346, 1.65573227885301,
	  1.97693148438101, 2.35308531443699, 2.61114717768636 },
	{ 0.676221397229444, 1.28754182284324, 1.65565516401972,
	  1.97681098932661, 2.35289497703906, 2.61089529178643 },
	{ 0.676209260433205, 1.28749973482824, 1.65557913475342,
	  1.97669219357061, 2.35270733034343, 2.61064697319431 },
	{ 0.67619729258047, 1.2874582341034, 1.65550416829193,
	  1.97657506141841, 2.35252231769728, 2.61040214666173 },
	{ 0.676185490168393, 1.28741730846127, 1.65543024250472,
	  1.97645955816809, 2.35233988402549, 2.61016073904242 },
	{ 0.676173849789814, 1.28737694603077, 1.65535733587174,
	  1.97634565007633, 2.35215997577884, 2.60992267922317 },
	{ 0.676162368130732, 1.2873371352651, 1.65528542746209,
	  1.97623330432526, 2.3519825408798, 2.60968789804268 },
	{ 0.676151041967024, 1.28729786493088, 1.65521449691403,
	  1.97612248899117, 2.35180752867303, 2.60945632823624 },
	{ 0.676139868160959, 1.28725912409796, 1.65514452935839,
	  1.97601317301423, 2.35163488987779, 2.6092279043645 },
	{ 0.676128843658924, 1.28722090212869, 1.65507549568695,
	  1.97590532616938, 2.3514645765393, 2.60900256275349 },
	{ 0.676117965488365, 1.28718318866856, 1.65500738201933,
	  1.97579891903848, 2.35129654198774, 2.60878024144009 },
	{ 0.676107230754936, 1.28714597363681, 1.65494017008626,
	  1.97569392298378, 2.35113074079231, 2.60856088010745 },
	{ 0.676096636640434, 1.28710924721719, 1.65487384209894,
	  1.9755903101217, 2.35096712872303, 2.60834442003147 },
	{ 0.676086180399593, 1.28707299984985, 1.65480838073355,
	  1.97548805329897, 2.35080566270913, 2.60813080403731 },
	{ 0.676075859358605, 1.28703722222232, 1.6547437691158,
	  1.97538712606793, 2.35064630080336, 2.60791997643982 },
	{ 0.676065670911754, 1.28700190526245, 1.65467999080702,
	  1.97528750266469, 2.35048900601105, 2.60771188299785 },
	{ 0.676055612520578, 1.28696704012965, 1.65461702978935,
	  1.97518915798624, 2.35033373081735, 2.60750647086652 },
	{ 0.67604568171043, 1.28693261820854, 1.65455487045327,
	  1.97509206757037, 2.35018044026671, 2.60730368855615 },
	{ 0.676035876069296, 1.28689863110118, 1.65449349758401,
	  1.97499620757449, 2.35002909655129, 2.60710348588701 },
	{ 0.676026193245616, 1.28686507062045, 1.65443289634941,
	  1.97490155475654, 2.34987966281774, 2.60690581394797 },
	{ 0.676016630946137, 1.28683192878367, 1.65437305228808,
	  1.97480808645661, 2.34973210313788, 2.60671062505399 },
	{ 0.676007186934724, 1.28679919780578, 1.65431395129748,
	  1.97471578057764, 2.34958638247728, 2.60651787271365 },
	{ 0.675997859029782, 1.28676687009392, 1.65425557962346,
	  1.97462461556946, 2.34944246667111, 2.60632751158888 },
	{ 0.6759886451034, 1.28673493824086, 1.65419792384896,
	  1.97453457041092, 2.34930032239436, 2.60613949745689 },
	{ 0.675979543079078, 1.28670340079362, 1.65414097088417,
	  1.97444562459431, 2.34915991713815, 2.60595378718067 },
	{ 0.675970550930727, 1.28667223922189, 1.65408470795636,
	  1.97435775810959, 2.34902121918331, 2.6057703386729 },
	{ 0.675961666680628, 1.2866414523479, 1.65402912260053,
	  1.97427095142927, 2.34888419757863, 2.60558911086576 },
	{ 0.675952888398199, 1.28661103345392, 1.6539742026502,
	  1.97418518549457, 2.34874882211561, 2.60541006367989 },
	{ 0.675944214199146, 1.28658097598145, 1.65391993622804,
	  1.97410044170017, 2.34861506330736, 2.60523315799297 },
	{ 0.675935642242836, 1.28655127352748, 1.65386631173825,
	  1.97401670188237, 2.348482892367, 2.6050583556172 },
	{ 0.675927170732366, 1.28652191983895, 1.65381331785739,
	  1.97393394830486, 2.34835228118798, 2.60488561926025 },
	{ 0.675918797912633, 1.28649290880884, 1.65376094352676,
	  1.97385216364663, 2.34822320232145, 2.60471491251394 },
	{ 0.675910522068663, 1.28646423447226, 1.65370917794513,
	  1.97377133099031, 2.3480956289606, 2.60454619981526 },
	{ 0.675902341525712, 1.28643589100141, 1.65365801056038,
	  1.97369143380953, 2.34796953491912, 2.60437944643025 },
	{ 0.675894254646936, 1.28640787270258, 1.6536074310632,
	  1.97361245595878, 2.34784489461493, 2.60421461842348 },
	{ 0.675886259832786, 1.28638017401192, 1.65355742937978,
	  1.97353438166157, 2.34772168305236, 2.60405168264158 },
	{ 0.675878355520225, 1.28635278949156, 1.65350799566496,
	  1.9734571955004, 2.34759987580556, 2.60389060668679 },
	{ 0.675870540180941, 1.28632571382667, 1.6534591202963,
	  1.97338088240676, 2.34747944900212, 2.60373135890042 },
	{ 0.675862812321167, 1.28629894182136, 1.65341079386722,
	  1.97330542765091, 2.34736037930823, 2.60357390833071 },
	{ 0.67585517048042, 1.28627246839562, 1.65336300718141,
	  1.97323081683251, 2.3472426439121, 2.60341822472614 },
	{ 0.675847613230231, 1.28624628858235, 1.65331575124682,
	  1.973157035872, 2.34712622051134, 2.60326427850791 },
	{ 0.675840139173983, 1.28622039752375, 1.65326901726996,
	  1.97308407100066, 2.34701108729665, 2.6031120407604 },
	{ 0.675832746945348, 1.28619479046878, 1.65322279665059,
	  1.97301190875303, 2.34689722294076, 2.60296148319442 },
	{ 0.675825435207774, 1.28616946277004, 1.65317708097644,
	  1.97294053595838, 2.34678460658268, 2.60281257815062 },
	{ 0.675818202653903, 1.28614440988085, 1.65313186201785,
	  1.97286993973233, 2.34667321781684, 2.60266529857165 },
	{ 0.675811048004202, 1.28611962735284, 1.65308713172325,
	  1.97280010746944, 2.34656303668027, 2.60251961798334 },
	{ 0.675803970006908, 1.28609511083293, 1.65304288221394,
	  1.97273102683559, 2.34645404363989, 2.60237551049206 },
	{ 0.675796967436522, 1.28607085606128, 1.65299910577993,
	  1.97266268576084, 2.34634621958206, 2.6022329507539 },
	{ 0.675790039093937, 1.2860468588682, 1.65295579487494,
	  1.9725950724322, 2.34623954580157, 2.60209191396492 },
	{ 0.675783183804925, 1.28602311517242, 1.6529129421126,
	  1.97252817528711, 2.34613400398875, 2.60195237585242 },
	{ 0.675776400420561, 1.28599962097796, 1.65287054026162,
	  1.97246198300614, 2.34602957622077, 2.60181431265503 },
	{ 0.675769687814895, 1.28597637237308, 1.65282858224262,
	  1.9723964845081, 2.34592624495284, 2.60167770111246 },
	{ 0.675763044886368, 1.28595336552668, 1.65278706112307,
	  1.97233166894183, 2.34582399300418, 2.60154251844985 },
	{ 0.675756470555385, 1.28593059668748, 1.65274597011456,
	  1.97226752568206, 2.3457228035529, 2.60140874236558 },
	{ 0.675749963764918, 1.28590806218115, 1.65270530256831,
	  1.9722040443224, 2.34562266012432, 2.60127635101916 },
	{ 0.67574352347974, 1.28588575840845, 1.65266505197184,
	  1.97214121467001, 2.34552354658223, 2.60114532302526 },
	{ 0.675737148685112, 1.28586368184388, 1.65262521194576,
	  1.97207902674079, 2.34542544712109, 2.60101563742984 },
	{ 0.675730838387242, 1.28584182903313, 1.65258577624003,
	  1.97201747075304, 2.34532834625705, 2.60088727370888 },
	{ 0.675724591612352, 1.28582019659155, 1.6525467387307,
	  1.97195653712303, 2.34523222881952, 2.60076021175596 },
	{ 0.675718407405964, 1.28579878120251, 1.65250809341712,
	  1.9718962164599, 2.34513707994377, 2.60063443186592 },
	{ 0.675712284832702, 1.28577757961561, 1.65246983441846,
	  1.97183649956073, 2.34504288506391, 2.60050991473612 },
	{ 0.675706222975847, 1.28575658864492, 1.65243195597087,
	  1.97177737740584, 2.34494962990386, 2.60038664144417 },
	{ 0.675700220936515, 1.28573580516768, 1.65239445242473,
	  1.97171884115455, 2.34485730047105, 2.60026459344611 },
	{ 0.675694277833993, 1.28571522612221, 1.65235731824137,
	  1.97166088648702, 2.34476588304919, 2.6001437525644 },
	{ 0.675688392804019, 1.28569484850729, 1.65232054799123,
	  1.97160349623738, 2.34467536419278, 2.60002410097684 },
	{ 0.675682564999789, 1.28567466937973, 1.65228413635015,
	  1.97154666639791, 2.34458573071702, 2.59990562121412 },
	{ 0.675676793590669, 1.2856546858535, 1.65224807809749,
	  1.97149038879973, 2.34449696969683, 2.59978829614542 },
	{ 0.675671077761967, 1.28563489509836, 1.6522123681135,
	  1.9714346554323, 2.34440906845472, 2.5996721089698 },
	{ 0.675665416714545, 1.28561529433839, 1.65217700137691,
	  1.97137945843963, 2.344322014559, 2.59955704322004 },
	{ 0.675659809664794, 1.28559588085057, 1.65214197296229,
	  1.97132479011532, 2.3442357958162, 2.59944308273501 },
	{ 0.675654255843693, 1.28557665196384, 1.65210727803823,
	  1.97127064290054, 2.34415040026581, 2.5993302116635 },
	{ 0.675648754496931, 1.28555760505755, 1.65207291186468,
	  1.97121700937969, 2.34406581617511, 2.59921841446216 },
	{ 0.675643304884117, 1.28553873756067, 1.65203886979129,
	  1.97116388227737, 2.3439820320325, 2.5991076758764 },
	{ 0.675637906279248, 1.28552004694996, 1.65200514725459,
	  1.97111125445455, 2.3438990365429, 2.59899798094287 },
	{ 0.675632557969187, 1.28550153074973, 1.6519717397768,
	  1.97105911890613, 2.34381681862333, 2.59888931497794 },
	{ 0.675627259254599, 1.28548318652991, 1.65193864296307,
	  1.97100746875719, 2.34373536739879, 2.59878166356806 },
	{ 0.675622009448735, 1.28546501190554, 1.65190585250014,
	  1.97095629726048, 2.343654672193, 2.59867501257294 },
	{ 0.675616807877367, 1.2854470045357, 1.65187336415422,
	  1.97090559779293, 2.34357472253002, 2.5985693481116 },
	{ 0.67561165387923, 1.28542916789241, 1.65184117376889,
	  1.97085536385333, 2.34349550812473, 2.59846465655791 },
	{ 0.675606546804178, 1.28541148822103, 1.65180927726407,
	  1.97080558905936, 2.34341701888108, 2.5983609245385 },
	{ 0.67560148601439, 1.28539396903422, 1.65177767063364,
	  1.97075626714457, 2.34333924488716, 2.59825813892028 },
	{ 0.675596470883723, 1.28537660815656, 1.65174634994371,
	  1.97070739195608, 2.343262176411, 2.59815628681403 },
	{ 0.675591500796958, 1.28535940345193, 1.6517153113316,
	  1.97065895745217, 2.34318580389501, 2.59805535555309 },
	{ 0.675586575149844, 1.28534235282254, 1.65168455100379,
	  1.97061095769942, 2.34311011795546, 2.59795533271572 },
	{ 0.675581693349097, 1.28532545420783, 1.65165406523442,
	  1.97056338687063, 2.34303510937435, 2.59785620608317 },
	{ 0.675576854811654, 1.28530870558405, 1.65162385036402,
	  1.97051623924199, 2.34296076910085, 2.59775796721621 },
	{ 0.675572058965392, 1.2852921049627, 1.65159390279742,
	  1.97046950919121, 2.34288708824096, 2.59766059725515 },
	{ 0.675567305247352, 1.28527565039101, 1.6515642190034,
	  1.97042319119527, 2.34281405806208, 2.59756408816459 },
	{ 0.675562593105119, 1.28525933994978, 1.65153479551214,
	  1.97037727982797, 2.34274166998119, 2.59746842857409 },
	{ 0.675557921995413, 1.28524317175377, 1.65150562891489,
	  1.97033176975802, 2.3426699155678, 2.59737360732246 },
	{ 0.675553291384827, 1.28522714395005, 1.65147671586164,
	  1.97028665574684, 2.34259878653715, 2.59727961343347 },
	{ 0.675548700748772, 1.28521125471807, 1.65144805306079,
	  1.97024193264691, 2.34252827475047, 2.59718643612561 },
	{ 0.67554414957206, 1.28519550226834, 1.65141963727724,
	  1.97019759539904, 2.34245837220623, 2.59709406480634 },
	{ 0.675539637347776, 1.28517988484241, 1.65139146533163,
	  1.97015363903158, 2.34238907104405, 2.59700248906172 },
	{ 0.675535163578226, 1.28516440071147, 1.65136353409863,
	  1.97011005865742, 2.34232036353504, 2.59691169865536 },
	{ 0.675530727773976, 1.2851490481762, 1.65133584050627,
	  1.97006684947286, 2.342252242085, 2.59682168353283 },
	{ 0.675526329453376, 1.28513382556641, 1.65130838153482,
	  1.97002400675592, 2.34218469922654, 2.59673243380325 },
	{ 0.675521968143782, 1.28511873123941, 1.65128115421502,
	  1.96998152586389, 2.34211772762006, 2.59664393974548 },
	{ 0.675517643379798, 1.28510376358055, 1.65125415562792,
	  1.96993940223254, 2.34205132004934, 2.59655619180422 },
	{ 0.675513354703935, 1.28508892100207, 1.65122738290324,
	  1.96989763137403, 2.34198546941777, 2.59646918058293 },
	{ 0.675509101666549, 1.28507420194249, 1.65120083321851,
	  1.96985620887516, 2.34192016874904, 2.59638289684233 },
	{ 0.675504883824932, 1.28505960486654, 1.65117450379823,
	  1.96981513039638, 2.34185541118186, 2.59629733149597 },
	{ 0.675500700744305, 1.28504512826386, 1.65114839191253,
	  1.96977439166956, 2.34179118996959, 2.59621247561132 },
	{ 0.675496551996593, 1.28503077064922, 1.65112249487668,
	  1.96973398849715, 2.34172749847533, 2.59612832039882 },
	{ 0.675492437160761, 1.28501653056167, 1.65109681004987,
	  1.96969391675038, 2.3416643301721, 2.59604485721764 },
	{ 0.675488355822988, 1.28500240656382, 1.65107133483421,
	  1.96965417236775, 2.34160167864096, 2.59596207756554 },
	{ 0.675484307575437, 1.28498839724218, 1.65104607157521,
	  1.9696147513543, 2.34153953756535, 2.59587997308106 },
	{ 0.675480292017706, 1.28497450120546, 1.65102100797974,
	  1.96957564977929, 2.34147790073243, 2.59579853553698 },
	{ 0.67547630875521, 1.28496071708525, 1.65099614645233,
	  1.96953686377581, 2.3414167620298, 2.59571775683901 },
	{ 0.675472357400064, 1.28494704353482, 1.65097148455905,
	  1.96949838953888, 2.34135611544239, 2.59563762902718 },
	{ 0.675468437570143, 1.28493347922929, 1.65094701990512,
	  1.9694602233247, 2.34129595505392, 2.59555814426264 },
	{ 0.6754645488901, 1.28492002286431, 1.65092275013348,
	  1.96942236144877, 2.34123627503916, 2.59547929483858 },
	{ 0.6754606909895, 1.28490667315701, 1.65089867292518,
	  1.96938480028569, 2.34117706966922, 2.5954010731653 },
	{ 0.675456863505007, 1.28489342884382, 1.65087478599712,
	  1.96934753626671, 2.34111833330226, 2.59532347177715 },
	{ 0.675453066077848, 1.28488028868188, 1.65085108710277,
	  1.96931056588008, 2.34106006038907, 2.59524648332597 },
	{ 0.675449298355577, 1.28486725144729, 1.65082757403044,
	  1.96927388566854, 2.34100224546641, 2.5951701005768 },
	{ 0.675445559990938, 1.28485431593548, 1.6508042446032,
	  1.96923749222922, 2.3409448831535, 2.59509431641024 },
	{ 0.675441850642157, 1.28484148096053, 1.65078109667792,
	  1.96920138221225, 2.34088796815834, 2.59501912381825 },
	{ 0.675438169972667, 1.2848287453549, 1.65075812814476,
	  1.96916555231957, 2.34083149526881, 2.59494451590036 },
	{ 0.675434517651202, 1.28481610796903, 1.65073533692629,
	  1.96912999930423, 2.34077545935307, 2.59487048586379 },
	{ 0.675430893351509, 1.28480356767108, 1.65071272097722,
	  1.9690947199692, 2.34071985535889, 2.59479702702057 },
	{ 0.675427296752262, 1.28479112334652, 1.65069027828348,
	  1.9690597111666, 2.3406646783119, 2.59472413278512 },
	{ 0.67542372753739, 1.28477877389758, 1.65066800686161,
	  1.96902496979629, 2.34060992331367, 2.59465179667388 },
	{ 0.675420185394851, 1.28476651824375, 1.65064590475881,
	  1.9689904928059, 2.3405555855399, 2.59458001230142 },
	{ 0.67541667001817, 1.28475435532021, 1.65062397005127,
	  1.96895627718883, 2.34050166024013, 2.59450877337731 },
	{ 0.675413181104833, 1.28474228407867, 1.6506022008446,
	  1.96892231998408, 2.34044814273569, 2.59443807371116 },
	{ 0.675409718357568, 1.28473030348611, 1.6505805952725,
	  1.96888861827517, 2.3403950284183, 2.59436790720054 },
	{ 0.675406281482657, 1.28471841252558, 1.65055915149717,
	  1.96885516918948, 2.34034231274834, 2.59429826783931 },
	{ 0.675402870191447, 1.28470661019481, 1.65053786770779,
	  1.96882196989706, 2.34028999125547, 2.59422914970806 },
	{ 0.675399484199175, 1.28469489550667, 1.65051674212069,
	  1.9687890176104, 2.34023805953427, 2.59416054697637 },
	{ 0.675396123225582, 1.28468326748843, 1.65049577297844,
	  1.96875630958291, 2.3401865132458, 2.59409245390125 },
	{ 0.675392786994637, 1.28467172518165, 1.65047495854951,
	  1.96872384310875, 2.34013534811459, 2.59402486482537 },
	{ 0.675389475233587, 1.28466026764252, 1.65045429712828,
	  1.96869161552213, 2.34008455992956, 2.59395777417332 },
	{ 0.675386187674592, 1.28464889394033, 1.65043378703352,
	  1.96865962419588, 2.34003414453947, 2.59389117645114 },
	{ 0.675382924053007, 1.28463760315851, 1.65041342660901,
	  1.9686278665418, 2.33998409785509, 2.59382506624636 },
	{ 0.675379684108913, 1.28462639439313, 1.65039321422206,
	  1.96859634000852, 2.3399344158463, 2.59375943822629 },
	{ 0.675376467584823, 1.2846152667543, 1.65037314826443,
	  1.96856504208276, 2.3398850945424, 2.5936942871341 },
	{ 0.675373274228249, 1.28460421936406, 1.65035322715033,
	  1.96853397028656, 2.3398361300286, 2.59362960779106 },
	{ 0.675370103789623, 1.28459325135757, 1.65033344931721,
	  1.96850312217818, 2.33978751844754, 2.59356539509033 },
	{ 0.67536695602304, 1.28458236188226, 1.65031381322483,
	  1.9684724953508, 2.3397392559973, 2.59350164400216 },
	{ 0.675363830686323, 1.28457155009763, 1.65029431735485,
	  1.9684420874319, 2.33969133892975, 2.59343834956329 },
	{ 0.675360727540468, 1.2845608151753, 1.65027496021068,
	  1.96841189608292, 2.33964376355098, 2.59337550688875 },
	{ 0.675357646350035, 1.28455015629844, 1.65025574031685,
	  1.96838191899833, 2.33959652621942, 2.59331311115615 },
	{ 0.675354586882825, 1.28453957266193, 1.65023665621879,
	  1.96835215390545, 2.33954962334423, 2.59325115761556 },
	{ 0.675351548909733, 1.28452906347199, 1.65021770648248,
	  1.96832259856342, 2.33950305138638, 2.5931896415824 },
	{ 0.675348532205426, 1.28451862794559, 1.65019888969372,
	  1.96829325076267, 2.33945680685571, 2.59312855844011 },
	{ 0.675345536546988, 1.28450826531118, 1.65018020445854,
	  1.96826410832514, 2.33941088631167, 2.59306790363289 },
	{ 0.67534256171505, 1.28449797480761, 1.65016164940205,
	  1.96823516910257, 2.33936528636121, 2.59300767267349 },
	{ 0.675339607493267, 1.28448775568432, 1.65014322316851,
	  1.96820643097678, 2.33932000365871, 2.59294786113289 },
	{ 0.675336673668199, 1.28447760720116, 1.65012492442102,
	  1.96817789185896, 2.33927503490496, 2.59288846464664 },
	{ 0.675333760028986, 1.28446752862849, 1.65010675184126,
	  1.96814954968918, 2.33923037684637, 2.59282947890956 },
	{ 0.675330866368459, 1.28445751924602, 1.65008870412858,
	  1.96812140243566, 2.33918602627418, 2.59277089967584 },
	{ 0.675327992481858, 1.28444757834369, 1.65007078000043,
	  1.96809344809449, 2.33914198002302, 2.59271272275572 },
	{ 0.675325138166532, 1.28443770522163, 1.6500529781921,
	  1.96806568468949, 2.33909823497241, 2.59265494402029 },
	{ 0.675322303224119, 1.28442789918827, 1.65003529745519,
	  1.96803811027073, 2.33905478804247, 2.59259755939414 },
	{ 0.675319487457372, 1.28441815956268, 1.65001773655924,
	  1.96801072291544, 2.33901163619627, 2.59254056486172 },
	{ 0.675316690672701, 1.28440848567229, 1.65000029428969,
	  1.96798352072628, 2.33896877643772, 2.59248395645746 },
	{ 0.67531391267878, 1.28439887685385, 1.64998296944862,
	  1.96795650183188, 2.33892620581179, 2.59242773026931 },
	{ 0.67531115328691, 1.28438933245298, 1.6499657608541,
	  1.96792966438569, 2.33888392140171, 2.59237188244046 },
	{ 0.675308412310666, 1.28437985182421, 1.6499486673402,
	  1.9679030065662, 2.33884192033129, 2.59231640916404 },
	{ 0.675305689566562, 1.28437043433031, 1.64993168775622,
	  1.96787652657588, 2.33880019976188, 2.59226130668447 },
	{ 0.675302984872972, 1.28436107934298, 1.64991482096711,
	  1.96785022264141, 2.33875875689276, 2.59220657129881 },
	{ 0.675300298051417, 1.28435178624164, 1.6498980658524,
	  1.96782409301265, 2.33871758896103, 2.59215219934713 },
	{ 0.675297628924947, 1.28434256016339, 1.64988142130689,
	  1.96779813596287, 2.33867669323928, 2.59209818722533 },
	{ 0.675294977319489, 1.28433338902991, 1.64986488623961,
	  1.96777234978791, 2.33863606703707, 2.5920445313719 },
	{ 0.675292343063037, 1.28432427797042, 1.649848459574,
	  1.9677467328061, 2.33859570769819, 2.59199122827254 },
	{ 0.675289725986224, 1.28431522639624, 1.6498321402473,
	  1.96772128335738, 2.33855561260221, 2.59193827446001 },
	{ 0.675287125920859, 1.28430623372725, 1.64981592721133,
	  1.96769599980399, 2.33851577916254, 2.59188566651383 },
	{ 0.675284542702256, 1.2842972993898, 1.64979981943055,
	  1.96767088052891, 2.33847620482599, 2.59183340105502 },
	{ 0.675281976166938, 1.28428842281842, 1.64978381588353,
	  1.96764592393621, 2.33843688707268, 2.59178147475114 },
	{ 0.675279426153597, 1.28427960345479, 1.64976791556189,
	  1.96762112845085, 2.33839782341582, 2.59172988431132 },
	{ 0.675276892503656, 1.28427084074733, 1.64975211746983,
	  1.96759649251746, 2.33835901139926, 2.59167862648681 },
	{ 0.675274375059767, 1.28426213415208, 1.64973642062483,
	  1.96757201460124, 2.33832044860045, 2.59162769807207 },
	{ 0.675271873667138, 1.28425348313165, 1.64972082405656,
	  1.96754769318663, 2.33828213262543, 2.59157709590119 },
	{ 0.675269388172612, 1.28424488715574, 1.64970532680726,
	  1.96752352677751, 2.33824406111342, 2.59152681685011 },
	{ 0.675266918425013, 1.28423634570069, 1.64968992793128,
	  1.96749951389691, 2.33820623173115, 2.59147685783184 },
	{ 0.675264464275559, 1.28422785824918, 1.64967462649469,
	  1.96747565308612, 2.33816864217643, 2.5914272158047 },
	{ 0.675262025576478, 1.28421942429099, 1.64965942157585,
	  1.96745194290558, 2.33813129017622, 2.59137788775783 },
	{ 0.675259602182669, 1.28421104332172, 1.64964431226419,
	  1.96742838193304, 2.33809417348493, 2.59132887072194 },
	{ 0.675257193950432, 1.28420271484363, 1.64962929766084,
	  1.96740496876455, 2.33805728988694, 2.59128016176623 },
	{ 0.675254800737544, 1.28419443836538, 1.64961437687831,
	  1.9673817020139, 2.33802063719238, 2.59123175799487 },
	{ 0.675252422404451, 1.28418621340114, 1.64959954903968,
	  1.96735858031161, 2.33798421324082, 2.59118365654783 },
	{ 0.675250058812841, 1.2841780394714, 1.64958481327917,
	  1.96733560230555, 2.33794801589661, 2.59113585460133 },
	{ 0.675247709825514, 1.28416991610307, 1.64957016874211,
	  1.96731276666049, 2.33791204305219, 2.59108834936999 },
	{ 0.675245375308051, 1.28416184282806, 1.6495556145837,
	  1.96729007205737, 2.33787629262506, 2.59104113809555 },
	{ 0.675243055126775, 1.28415381918472, 1.64954114997007,
	  1.9672675171937, 2.33784076255937, 2.59099421806255 },
	{ 0.675240749150494, 1.28414584471648, 1.64952677407713,
	  1.96724510078246, 2.33780545082337, 2.5909475865784 },
	{ 0.675238457248578, 1.28413791897301, 1.64951248609142,
	  1.9672228215532, 2.33777035541109, 2.59090124099562 },
	{ 0.675236179293274, 1.28413004150862, 1.64949828520866,
	  1.96720067824991, 2.337735474341, 2.59085517868955 },
	{ 0.675233915156981, 1.28412221188401, 1.64948417063522,
	  1.96717866963301, 2.33770080565499, 2.59080939706908 },
	{ 0.675231664714837, 1.28411442966434, 1.64947014158628,
	  1.96715679447701, 2.3376663474196, 2.59076389358105 },
	{ 0.675229427842487, 1.28410669442073, 1.6494561972872,
	  1.96713505157185, 2.3376320977243, 2.59071866569759 },
	{ 0.6752272044181, 1.28409900572872, 1.64944233697197,
	  1.96711343972164, 2.33759805468116, 2.59067371091891 },
	{ 0.675224994320579, 1.28409136316948, 1.64942855988427,
	  1.96709195774507, 2.33756421642625, 2.59062902678084 },
	{ 0.675222797430061, 1.28408376632937, 1.64941486527698,
	  1.96707060447516, 2.33753058111713, 2.59058461084945 },
	{ 0.675220613629131, 1.28407621479891, 1.6494012524112,
	  1.96704937875845, 2.33749714693351, 2.59054046071197 },
	{ 0.675218442800856, 1.28406870817413, 1.64938772055747,
	  1.96702827945551, 2.33746391207626, 2.59049657399418 },
	{ 0.675216284829822, 1.28406124605579, 1.64937426899483,
	  1.96700730544044, 2.33743087476844, 2.5904529483418 },
	{ 0.675214139602406, 1.28405382804906, 1.64936089701065,
	  1.96698645560056, 2.33739803325423, 2.59040958143756 },
	{ 0.675212007005933, 1.28404645376406, 1.64934760390091,
	  1.96696572883632, 2.33736538579846, 2.5903664709796 },
	{ 0.675209886929392, 1.28403912281521, 1.64933438896965,
	  1.96694512406125, 2.33733293068517, 2.59032361470631 },
	{ 0.675207779262581, 1.28403183482181, 1.64932125152946,
	  1.96692464020149, 2.33730066622013, 2.59028101037213 },
	{ 0.675205683896818, 1.28402458940745, 1.64930819090068,
	  1.96690427619589, 2.33726859072904, 2.59023865576223 },
	{ 0.675203600725133, 1.28401738619978, 1.64929520641143,
	  1.96688403099537, 2.33723670255523, 2.59019654868953 },
	{ 0.675201529640776, 1.28401022483152, 1.64928229739818,
	  1.96686390356362, 2.33720500006373, 2.59015468698854 },
	{ 0.675199470539414, 1.28400310493879, 1.64926946320443,
	  1.96684389287572, 2.33717348163727, 2.59011306852135 },
	{ 0.675197423317175, 1.28399602616243, 1.64925670318171,
	  1.96682399791897, 2.33714214567655, 2.59007169117406 },
	{ 0.675195387871148, 1.28398898814759, 1.6492440166891,
	  1.96680421769246, 2.3371109906032, 2.59003055285665 },
	{ 0.675193364100864, 1.28398199054263, 1.64923140309238,
	  1.96678455120629, 2.33708001485384, 2.58998965150531 },
	{ 0.675191351905303, 1.2839750330011, 1.64921886176555,
	  1.96676499748245, 2.33704921688592, 2.5899489850792 },
	{ 0.675189351185966, 1.28396811517955, 1.649206392089,
	  1.9667455555539, 2.33701859517262, 2.58990855155735 },
	{ 0.675187361844818, 1.28396123673891, 1.64919399345048,
	  1.96672622446448, 2.33698814820517, 2.58986834894534 },
	{ 0.675185383784981, 1.28395439734391, 1.64918166524479,
	  1.96670700326911, 2.33695787449253, 2.58982837527058 },
	{ 0.675183416910849, 1.28394759666299, 1.64916940687338,
	  1.96668789103344, 2.3369277725599, 2.58978862858671 },
	{ 0.675181461127639, 1.28394083436846, 1.64915721774467,
	  1.96666888683352, 2.3368978409492, 2.58974910696173 },
	{ 0.675179516342045, 1.28393411013611, 1.64914509727348,
	  1.96664998975589, 2.33686807821913, 2.58970980848997 },
	{ 0.675177582461589, 1.28392742364539, 1.64913304488133,
	  1.96663119889725, 2.33683848294416, 2.58967073128813 },
	{ 0.675175659394627, 1.28392077457963, 1.64912105999637,
	  1.96661251336477, 2.33680905371528, 2.5896318734921 },
	{ 0.675173747050844, 1.28391416262539, 1.64910914205292,
	  1.96659393227514, 2.33677978913848, 2.58959323325865 },
	{ 0.675171845340602, 1.28390758747293, 1.64909729049177,
	  1.96657545475529, 2.3367506878368, 2.58955480876455 },
	{ 0.6751699541755, 1.28390104881587, 1.64908550475989,
	  1.96655707994181, 2.33672174844684, 2.58951659821387 },
	{ 0.67516807346826, 1.28389454635103, 1.64907378431014,
	  1.96653880698044, 2.33669296962108, 2.58947859981798 },
	{ 0.675166203132331, 1.28388807977879, 1.64906212860169,
	  1.96652063502667, 2.33666435002732, 2.58944081181711 },
	{ 0.675164343082038, 1.2838816488029, 1.64905053709966,
	  1.96650256324561, 2.33663588834754, 2.58940323247088 },
	{ 0.67516249323299, 1.28387525313004, 1.64903900927481,
	  1.96648459081096, 2.33660758327805, 2.58936586005197 },
	{ 0.675160653501124, 1.28386889247059, 1.64902754460413,
	  1.96646671690592, 2.33657943353002, 2.58932869285882 },
	{ 0.675158823803902, 1.28386256653766, 1.6490161425699,
	  1.96644894072245, 2.33655143782878, 2.5892917292055 },
	{ 0.675157004059428, 1.28385627504763, 1.64900480266023,
	  1.96643126146122, 2.33652359491341, 2.58925496742508 },
	{ 0.675155194186928, 1.28385001771982, 1.64899352436861,
	  1.96641367833167, 2.33649590353568, 2.58921840586375 },
	{ 0.675153394105649, 1.28384379427721, 1.64898230719461,
	  1.96639619055214, 2.33646836246247, 2.58918204289739 },
	{ 0.675151603737097, 1.28383760444473, 1.64897115064228,
	  1.96637879734871, 2.33644097047294, 2.58914587690653 },
	{ 0.675149823002531, 1.2838314479511, 1.64896005422174,
	  1.96636149795659, 2.33641372636037, 2.58910990629701 },
	{ 0.675148051824269, 1.28382532452782, 1.64894901744823,
	  1.96634429161872, 2.33638662893026, 2.58907412949073 },
	{ 0.675146290125747, 1.28381923390898, 1.648938039842,
	  1.96632717758631, 2.33635967700099, 2.58903854492395 },
	{ 0.675144537831132, 1.28381317583163, 1.64892712092847,
	  1.9663101551185, 2.33633286940418, 2.58900315105241 },
	{ 0.675142794865286, 1.28380715003574, 1.64891626023828,
	  1.96629322348244, 2.33630620498331, 2.588967946347 },
	{ 0.675141061153729, 1.28380115626406, 1.64890545730702,
	  1.96627638195316, 2.33627968259424, 2.58893292929601 },
	{ 0.675139336623187, 1.28379519426182, 1.64889471167504,
	  1.96625962981326, 2.33625330110581, 2.5888980984014 },
	{ 0.675137621200879, 1.28378926377712, 1.64888402288766,
	  1.9662429663528, 2.33622705939745, 2.5888634521822 },
	{ 0.675135914814729, 1.28378336456082, 1.64887339049515,
	  1.96622639086959, 2.33620095636146, 2.58882898917836 },
	{ 0.67513421739355, 1.28377749636619, 1.64886281405225,
	  1.96620990266865, 2.33617499090143, 2.5887947079354 },
	{ 0.675132528867003, 1.28377165894913, 1.6488522931185,
	  1.96619350106252, 2.33614916193277, 2.58876060702168 },
	{ 0.675130849164996, 1.28376585206841, 1.64884182725827,
	  1.96617718537095, 2.33612346838242, 2.58872668501527 },
	{ 0.675129178219146, 1.28376007548457, 1.64883141603979,
	  1.96616095492014, 2.33609790918718, 2.58869294051681 },
	{ 0.67512751596074, 1.28375432896144, 1.6488210590366,
	  1.96614480904421, 2.336072483297, 2.58865937213126 },
	{ 0.675125862322323, 1.28374861226491, 1.64881075582619,
	  1.96612874708374, 2.33604718967076, 2.58862597848816 },
	{ 0.675124217236749, 1.2837429251635, 1.64880050599075,
	  1.96611276838624, 2.33602202728004, 2.58859275822132 },
	{ 0.675122580638354, 1.28373726742769, 1.64879030911631,
	  1.96609687230588, 2.33599699510582, 2.58855970999066 },
	{ 0.675120952461135, 1.28373163883092, 1.64878016479367,
	  1.9660810582036, 2.33597209213956, 2.58852683245842 },
	{ 0.675119332640693, 1.28372603914835, 1.64877007261743,
	  1.96606532544676, 2.33594731738347, 2.58849412430484 },
	{ 0.675117721112038, 1.28372046815835, 1.64876003218699,
	  1.96604967340992, 2.33592266985052, 2.58846158422932 },
	{ 0.675116117812961, 1.28371492564002, 1.64875004310462,
	  1.96603410147247, 2.33589814856264, 2.58842921093369 },
	{ 0.67511452267952, 1.2837094113764, 1.64874010497807,
	  1.96601860902175, 2.33587375255269, 2.58839700314423 },
	{ 0.675112935650231, 1.28370392515151, 1.64873021741799,
	  1.96600319545054, 2.33584948086283, 2.5883649595917 },
	{ 0.675111356662303, 1.28369846675282, 1.64872038004016,
	  1.96598786015833, 2.33582533254551, 2.58833307902397 },
	{ 0.675109785656534, 1.28369303596792, 1.64871059246239,
	  1.96597260254937, 2.33580130666223, 2.58830136020232 },
	{ 0.675108222570836, 1.28368763258904, 1.6487008543086,
	  1.96595742203593, 2.33577740228419, 2.58826980189413 },
	{ 0.675106667346862, 1.28368225640821, 1.64869116520443,
	  1.96594231803444, 2.33575361849242, 2.58823840289015 },
	{ 0.675105119924088, 1.28367690722162, 1.64868152478115,
	  1.96592728996837, 2.33572995437645, 2.58820716198433 },
	{ 0.675103580245112, 1.28367158482562, 1.648671932672,
	  1.96591233726621, 2.33570640903457, 2.58817607798515 },
	{ 0.675102048251538, 1.28366628901973, 1.64866238851502,
	  1.9658974593625, 2.33568298157655, 2.58814514971423 },
	{ 0.675100523886035, 1.28366101960514, 1.64865289195149,
	  1.96588265569744, 2.33565967111767, 2.58811437600421 },
	{ 0.675099007091342, 1.2836557763854, 1.6486434426265,
	  1.96586792571699, 2.3356364767838, 2.58808375569749 },
	{ 0.675097497811899, 1.28365055916513, 1.6486340401881,
	  1.96585326887188, 2.33561339771059, 2.58805328765398 },
	{ 0.675095995991311, 1.28364536775191, 1.64862468428854,
	  1.96583868461915, 2.33559043304003, 2.58802297073721 },
	{ 0.675094501575394, 1.28364020195404, 1.64861537458257,
	  1.96582417242046, 2.33556758192341, 2.58799280382361 },
	{ 0.675093014507855, 1.28363506158362, 1.64860611072993,
	  1.96580973174394, 2.33554484352153, 2.58796278580889 },
	{ 0.675091534736502, 1.28362994645199, 1.64859689239158,
	  1.96579536206122, 2.33552221700119, 2.58793291559266 },
	{ 0.675090062206585, 1.28362485637459, 1.64858771923359,
	  1.96578106285057, 2.33549970153989, 2.58790319207953 },
	{ 0.675088596865264, 1.28361979116766, 1.64857859092455,
	  1.96576683359476, 2.33547729632098, 2.58787361419677 },
	{ 0.675087138660582, 1.28361475064897, 1.648569507136,
	  1.96575267378152, 2.33545500053742, 2.58784418087653 },
	{ 0.675085687539385, 1.28360973463928, 1.64856046754369,
	  1.96573858290417, 2.33543281338972, 2.58781489106028 },
	{ 0.675084243451869, 1.28360474295903, 1.64855147182492,
	  1.9657245604599, 2.33541073408443, 2.5877857436991 },
	{ 0.675082806345409, 1.28359977543284, 1.64854251966194,
	  1.96571060595191, 2.33538876183804, 2.58775673775821 },
	{ 0.675081376170281, 1.28359483188521, 1.64853361073872,
	  1.96569671888753, 2.33536689587422, 2.58772787221362 },
	{ 0.675079952876481, 1.28358991214299, 1.64852474474278,
	  1.96568289877878, 2.33534513542356, 2.58769914604396 },
	{ 0.67507853641425, 1.28358501603471, 1.6485159213647,
	  1.96566914514295, 2.33532347972308, 2.58767055824658 },
	{ 0.67507712673466, 1.28358014339044, 1.64850714029785,
	  1.96565545750133, 2.33530192801981, 2.58764210781818 },
	{ 0.675075723788858, 1.28357529404207, 1.64849840123873,
	  1.96564183538025, 2.33528047956526, 2.58761379377598 },
	{ 0.675074327529003, 1.28357046782271, 1.6484897038865,
	  1.96562827831027, 2.3352591336192, 2.58758561514328 },
	{ 0.675072937907357, 1.28356566456737, 1.64848104794328,
	  1.96561478582667, 2.33523788944943, 2.58755757094485 },
	{ 0.675071554876397, 1.28356088411268, 1.64847243311425,
	  1.96560135746889, 2.3352167463294, 2.58752966022427 },
	{ 0.675070178390146, 1.28355612629615, 1.64846385910671,
	  1.96558799278052, 2.33519570353918, 2.58750188203014 },
	{ 0.675068808401284, 1.28355139095805, 1.64845532563176,
	  1.96557469131056, 2.33517476036662, 2.58747423542094 },
	{ 0.675067444864126, 1.28354667793929, 1.64844683240253,
	  1.96556145261108, 2.33515391610709, 2.58744671946557 },
	{ 0.675066087733797, 1.28354198708198, 1.6484383791346,
	  1.96554827623867, 2.33513317006001, 2.5874193332385 },
	{ 0.675064736964884, 1.28353731823053, 1.64842996554691,
	  1.96553516175425, 2.3351125215336, 2.58739207582294 },
	{ 0.675063392512457, 1.2835326712306, 1.64842159136083,
	  1.9655221087233, 2.33509196984213, 2.58736494631384 },
	{ 0.675062054332767, 1.28352804592885, 1.64841325629994,
	  1.96550911671426, 2.33507151430575, 2.58733794381605 },
	{ 0.67506072238176, 1.2835234421738, 1.64840496009089,
	  1.96549618530063, 2.33505115425039, 2.58731106743537 },
	{ 0.675059396616449, 1.28351885981501, 1.64839670246238,
	  1.96548331405902, 2.33503088901014, 2.58728431629083 },
	{ 0.67505807699321, 1.28351429870407, 1.64838848314631,
	  1.96547050257109, 2.33501071792406, 2.58725768950777 },
	{ 0.675056763469699, 1.28350975869339, 1.64838030187655,
	  1.96545775042127, 2.33499064033735, 2.5872311862286 },
	{ 0.675055456004054, 1.28350523963667, 1.64837215838929,
	  1.96544505719811, 2.33497065560111, 2.58720480558776 },
	{ 0.67505415455418, 1.28350074138933, 1.64836405242361,
	  1.96543242249452, 2.33495076307349, 2.58717854673728 },
	{ 0.675052859078506, 1.2834962638081, 1.64835598372085,
	  1.96541984590656, 2.33493096211708, 2.58715240883818 },
	{ 0.675051569536321, 1.28349180675065, 1.64834795202448,
	  1.96540732703415, 2.33491125210105, 2.5871263910594 },
	{ 0.675050285886951, 1.28348737007619, 1.64833995708047,
	  1.9653948654808, 2.33489163240127, 2.58710049256822 },
	{ 0.675049008089658, 1.28348295364552, 1.64833199863739,
	  1.96538246085391, 2.33487210239678, 2.58707471254918 },
	{ 0.675047736104799, 1.28347855732023, 1.64832407644569,
	  1.96537011276447, 2.3348526614757, 2.58704905019047 },
	{ 0.675046469893262, 1.28347418096297, 1.64831619025799,
	  1.96535782082631, 2.33483330902861, 2.58702350468828 },
	{ 0.675045209414728, 1.28346982443883, 1.64830833983002,
	  1.96534558465792, 2.33481404445342, 2.58699807524975 },
	{ 0.675043954631535, 1.28346548761242, 1.64830052491838,
	  1.96533340387985, 2.33479486715235, 2.58697276108154 },
	{ 0.675042705504331, 1.28346117035106, 1.64829274528318,
	  1.96532127811756, 2.33477577653463, 2.58694756140298 },
	{ 0.675041461995333, 1.28345687252241, 1.64828500068583,
	  1.96530920699899, 2.33475677201407, 2.58692247544118 },
	{ 0.675040224066535, 1.28345259399561, 1.64827729089022,
	  1.96529719015548, 2.33473785300822, 2.5868975024256 },
	{ 0.675038991680864, 1.28344833464072, 1.64826961566209,
	  1.96528522722158, 2.3347190189421, 2.5868726415946 },
	{ 0.675037764800857, 1.28344409432936, 1.64826197476973,
	  1.96527331783563, 2.33470026924495, 2.58684789219579 },
	{ 0.675036543389522, 1.28343987293423, 1.64825436798329,
	  1.96526146163887, 2.33468160335089, 2.58682325348322 },
	{ 0.675035327410563, 1.28343567032888, 1.64824679507477,
	  1.96524965827578, 2.33466302069921, 2.58679872471063 },
	{ 0.675034116828095, 1.28343148638797, 1.64823925581831,
	  1.96523790739389, 2.33464452073455, 2.58677430514797 },
	{ 0.675032911605985, 1.2834273209876, 1.64823174999017,
	  1.96522620864386, 2.33462610290639, 2.58674999406738 },
	{ 0.67503171170832, 1.28342317400514, 1.64822427736879,
	  1.96521456168001, 2.33460776666811, 2.58672579074603 },
	{ 0.675030517100805, 1.28341904531801, 1.64821683773374,
	  1.96520296615871, 2.33458951147974, 2.58670169446941 },
	{ 0.675029327747648, 1.28341493480603, 1.64820943086761,
	  1.9651914217402, 2.33457133680565, 2.58667770452978 },
	{ 0.675028143615082, 1.28341084234891, 1.64820205655385,
	  1.96517992808714, 2.33455324211248, 2.58665382022303 },
	{ 0.675026964667936, 1.28340677355261, 1.6481947145789,
	  1.96516848486579, 2.33453522687467, 2.58663004085564 },
	{ 0.675025790873073, 1.28340271686189, 1.64818740473,
	  1.9651570917445, 2.33451729056878, 2.58660636573587 },
	{ 0.675024622196244, 1.28339867787341, 1.64818012679702,
	  1.9651457483953, 2.33449943267968, 2.58658279417991 },
	{ 0.675023458604113, 1.28339465647145, 1.64817288057143,
	  1.96513445449267, 2.3344816526918, 2.58655932550919 },
	{ 0.675022300063668, 1.28339065254139, 1.64816566584634,
	  1.96512320971392, 2.33446395009751, 2.5865359590533 },
	{ 0.675021146541882, 1.28338666596973, 1.64815848241697,
	  1.96511201373953, 2.33444632439293, 2.5865126941444 },
	{ 0.675019998006743, 1.28338269664342, 1.64815133007978,
	  1.96510086625184, 2.33442877507748, 2.58648953011909 },
	{ 0.675018854425576, 1.28337874445115, 1.64814420863358,
	  1.96508976693713, 2.33441130165583, 2.58646646633158 },
	{ 0.675017715766327, 1.28337480928222, 1.6481371178787,
	  1.96507871548367, 2.33439390363659, 2.58644350212345 },
	{ 0.675016581997441, 1.2833708910268, 1.64813005761715,
	  1.96506771158255, 2.33437658053351, 2.58642063685441 },
	{ 0.675015453087491, 1.28336698957607, 1.64812302765258,
	  1.96505675492747, 2.33435933186219, 2.58639786988955 },
	{ 0.675014329005152, 1.28336310482226, 1.64811602779049,
	  1.96504584521484, 2.33434215714586, 2.58637520059147 },
	{ 0.675013209720034, 1.28335923665805, 1.64810905783766,
	  1.96503498214342, 2.33432505590938, 2.58635262833487 },
	{ 0.675012095200489, 1.2833553849781, 1.64810211760339,
	  1.96502416541523, 2.33430802768068, 2.58633015249531 },
	{ 0.675010985417315, 1.28335154967647, 1.64809520689732,
	  1.96501339473376, 2.33429107199556, 2.58630777246175 },
	{ 0.67500988033963, 1.28334773064939, 1.64808832553182,
	  1.965002669806, 2.33427418838983, 2.58628548762024 },
	{ 0.675008779937405, 1.28334392779367, 1.64808147332058,
	  1.96499199034108, 2.33425737640518, 2.5862632973648 },
	{ 0.675007684182012, 1.28334014100606, 1.6480746500781,
	  1.96498135605009, 2.33424063558615, 2.58624120109459 },
	{ 0.675006593043444, 1.28333637018542, 1.64806785562145,
	  1.96497076664727, 2.33422396548387, 2.58621919821324 },
	{ 0.6750055064921, 1.28333261523132, 1.64806108976913,
	  1.96496022184926, 2.33420736564875, 2.58619728813148 },
	{ 0.675004424499913, 1.28332887604327, 1.64805435234043,
	  1.96494972137447, 2.3341908356393, 2.58617547026194 },
	{ 0.675003347037913, 1.28332515252237, 1.64804764315678,
	  1.96493926494415, 2.33417437501586, 2.5861537440239 },
	{ 0.675002274077966, 1.28332144457023, 1.64804096204082,
	  1.96492885228165, 2.33415798334054, 2.58613210884117 },
	{ 0.675001205591441, 1.28331775208973, 1.64803430881705,
	  1.964918483113, 2.33414166018354, 2.58611056414294 },
	{ 0.675000141550741, 1.28331407498397, 1.64802768331094,
	  1.96490815716602, 2.33412540511416, 2.58608910936278 },
	{ 0.674999081927839, 1.28331041315739, 1.64802108534989,
	  1.96489787417129, 2.33410921770812, 2.58606774393894 },
	{ 0.674998026696166, 1.28330676651435, 1.64801451476196,
	  1.96488763386088, 2.33409309754341, 2.58604646731469 },
	{ 0.674996975827084, 1.28330313496159, 1.64800797137798,
	  1.9648774359703, 2.33407704420306, 2.58602527893618 },
	{ 0.674995929295134, 1.28329951840449, 1.6480014550284,
	  1.96486728023567, 2.33406105727092, 2.58600417825726 },
	{ 0.674994887072205, 1.2832959167514, 1.64799496554683,
	  1.96485716639689, 2.33404513633653, 2.58598316473442 },
	{ 0.674993849132717, 1.28329232990958, 1.64798850276689,
	  1.96484709419469, 2.33402928099175, 2.5859622378263 },
	{ 0.674992815449735, 1.28328875778832, 1.64798206652443,
	  1.96483706337281, 2.33401349083265, 2.58594139700235 },
	{ 0.674991785997126, 1.28328520029717, 1.64797565665632,
	  1.96482707367674, 2.33399776545832, 2.58592064172968 },
	{ 0.674990760749498, 1.28328165734607, 1.64796927300049,
	  1.96481712485377, 2.33398210447009, 2.58589997148233 },
	{ 0.674989739680223, 1.28327812884671, 1.64796291539705,
	  1.96480721665415, 2.33396650747286, 2.58587938574209 },
	{ 0.674988722764844, 1.28327461471013, 1.64795658368616,
	  1.96479734882893, 2.3339509740761, 2.58585888398766 },
	{ 0.674987709977448, 1.28327111484919, 1.64795027771036,
	  1.96478752113221, 2.33393550389198, 2.58583846570912 },
	{ 0.674986701292957, 1.28326762917713, 1.64794399731313,
	  1.96477773331965, 2.33392009653436, 2.5858181303968 },
	{ 0.674985696686923, 1.28326415760751, 1.64793774233878,
	  1.96476798514884, 2.33390475162133, 2.58579787754609 },
	{ 0.674984696134111, 1.28326070005534, 1.64793151263371,
	  1.96475827637959, 2.33388946877487, 2.58577770665583 },
	{ 0.674983699610458, 1.28325725643554, 1.64792530804471,
	  1.96474860677329, 2.33387424761908, 2.58575761723022 },
	{ 0.67498270709127, 1.28325382666439, 1.64791912842056,
	  1.96473897609379, 2.33385908778001, 2.58573760877641 },
	{ 0.6749817185532, 1.28325041065799, 1.64791297361037,
	  1.96472938410599, 2.33384398888902, 2.58571768080559 },
	{ 0.674980733971083, 1.28324700833451, 1.64790684346575,
	  1.96471983057788, 2.3338289505796, 2.58569783283286 } };
    
}
